/**********************************************************************/
/* ASPECT DLL Template                                                */
/**********************************************************************/

#include <windows.h>
#include <stdarg.h>

#include <stdio.h>                     /* for HelloWorld example */

/**********************************************************************/
/* Common Definitions                                                 */
/**********************************************************************/

/* Required for ASPECT-callable DLL functions */
#define EXPORTED      __declspec(dllexport)

/* ASPECT data type encoding */
#define ASP_STRING    0
#define ASP_INTEGER   1
#define ASP_LONG      2
#define ASP_FLOAT     3

/* Maximum length of an ASPECT string */
#define ASPSTRSIZE    256

/**********************************************************************/
/* Function Argument Reference Macros                                 */
/**********************************************************************/

#define P_ASP_ARG(type, index)  ((type *)vdatptr[index])
#define ASP_ARG(type, index)    (*P_ASP_ARG(type, index))

/* Obtain value of various argument type */
#define STR_ARG(index)       ((LPSTR)vdatptr[index])
#define INT_ARG(index)       ASP_ARG(int, index)
#define LONG_ARG(index)      ASP_ARG(long, index)
#define FLOAT_ARG(index)     ASP_ARG(double, index)

/* Obtain pointer to various argument type */
#define P_INT_ARG(index)     P_ASP_ARG(int, index)
#define P_LONG_ARG(index)    P_ASP_ARG(long, index)
#define P_FLOAT_ARG(index)   P_ASP_ARG(double, index)

/* Examples of extending functionality to other common Windows types */
#define UINT_ARG(index)      ASP_ARG(unsigned int, index)
#define ULONG_ARG(index)     ASP_ARG(unsigned long, index)
#define DWORD_ARG(index)     ASP_ARG(DWORD, index)
#define BYTE_ARG(index)      ASP_ARG(BYTE, index)
#define LPBYTE_ARG(index)    ASP_ARG(LPBYTE, index)
#define HANDLE_ARG(index)    ASP_ARG(HANDLE, index)
#define HINST_ARG(index)     ASP_ARG(HINSTANCE, index)
#define HWND_ARG(index)      ASP_ARG(HWND, index)

/**********************************************************************/
/* Globals                                                            */
/**********************************************************************/

HANDLE hInst;

/**********************************************************************/
/* Main Entry Point                                                   */
/**********************************************************************/

BOOL WINAPI DllMain(HANDLE hModule, DWORD dwFunction, LPVOID lpNot)
{
   switch(dwFunction)
   {
      case DLL_PROCESS_ATTACH:
         hInst = hModule;
         break;
      case DLL_PROCESS_DETACH:
         break;
   }
   return(TRUE);
}

/**********************************************************************/
/* Local Functions                                                    */
/**********************************************************************/

BOOL __cdecl CheckArgTypes(int argcnt, LPBYTE vtypeptr, int argsreqd, ...)
{
   va_list arglist;
   int i;

   if(argcnt < argsreqd)
      return(FALSE);
   va_start(arglist, argsreqd);
   for(i=0; i<argsreqd; ++i)
      if(vtypeptr[i] != va_arg(arglist, int))
         return(FALSE);
   return(TRUE);
}

/**********************************************************************/
/* Exported APIs                                                      */
/**********************************************************************/

EXPORTED BOOL CALLBACK HelloWorld(HWND hWnd, HANDLE hPWInst, LPVOID FAR *vdatptr, LPBYTE vtypeptr, int argcnt)
{
   UCHAR buffer[256];

   /* Args: String, Integer, Long, Float */
   if(!CheckArgTypes(argcnt, vtypeptr, 4, ASP_STRING, ASP_INTEGER, ASP_LONG, ASP_FLOAT))
      return(FALSE);

   /* Format and display values passed in */
   sprintf(buffer, "Args: %s, %d, %ld, %f", STR_ARG(0), INT_ARG(1), LONG_ARG(2), FLOAT_ARG(3));
   MessageBox(hWnd, buffer, "Hello, World!", MB_OK);

   /* Change the arguments to demonstrate returning values to ASPECT */
   lstrcpy(STR_ARG(0), "Done");
   *P_INT_ARG(1) = 1;
   *P_LONG_ARG(2) = 2L;
   *P_FLOAT_ARG(3) = 3.4;

   return(TRUE);
}

